--GET THE DISTANCE BETWEEN TWO SETS OF COORDS:
function Distance(x, y, x2, y2)
    return math.sqrt(((x-x2)^2) + ((y-y2)^2))
end--> distance


--ROTATE A SET OF COORDS AROUND ANOTHER SET OF COORDS:
function rotatePoint(x,y,cx,cy,rads)
    --local angle = radToDeg(rads)
    local sin = math.sin(rads)
    local cos = math.cos(rads)
    x = x - cx
    y = y - cy
    local new_x = (x * cos) - (y * sin)
    local new_y = (x * sin) - (y * cos)
    x = new_x + cx
    y = new_y + cy
    return x, y
end--> rotated x, y

--MOVE A SET OF COORDS A SPECIFIC DISTANCE TOWARD ANOTHER SET OF COORDS:
function MoveCoordsAlongVector(x, y, x2, y2, dist)
    local dist2 = math.sqrt(((x-x2)^2) + ((y-y2)^2))
    local x3 = x + (((x2-x)/dist2)*dist)
    local y3 = y + (((y2-y)/dist2)*dist)
    return x3, y3
end--> moved x, y

--APPLY FORCE TO AN ENTITY:
function EntityApplyForce(entity, force_x, force_y)
    local x, y = EntityGetTransform(entity)
    local chardatacomp = EntityGetFirstComponent(entity, "CharacterDataComponent")
    local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
    local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity, "PhysicsBody2Component") or EntityGetFirstComponent(entity, "SimplePhysicsComponent")
    local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
    local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")

    if physcomp or not proj_vel_x then
        PhysicsApplyForce(entity, force_x, force_y)
    end
    if vel_x then 
        ComponentSetValue2(chardatacomp, "mVelocity", vel_x+force_x, vel_y+force_y)
    elseif proj_vel_x then
        ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+force_x, proj_vel_y+force_y)
    end
end

--CONSTRAIN AN ENTITY TO A SET OF COORDS:
function constrain(entity, x2, y2, max_dist)
    local chardatacomp = EntityGetFirstComponent(entity, "CharacterDataComponent")
    local projcomp = EntityGetFirstComponent(entity, "VelocityComponent")
    local physcomp = EntityGetFirstComponent(entity, "PhysicsBodyComponent") or EntityGetFirstComponent(entity1, "PhysicsBody2Component") or EntityGetFirstComponent(entity1, "SimplePhysicsComponent")
    local vel_x, vel_y = ComponentGetValue2(chardatacomp, "mVelocity")
    local proj_vel_x, proj_vel_y = ComponentGetValue2(projcomp, "mVelocity")
    local x1, y1 = EntityGetTransform(entity)
    if (not x1) or (not x2) then return nil end
        
    --CALCULATE CONSTRAINT FORCE:
    local dist = math.sqrt(((x2-x1)^2) + ((y2-y1)^2))
    local target_x = x2 + (((x1-x2)/dist)*max_dist)
    local target_y = y2 + (((y1-y2)/dist)*max_dist)
    local target_vel_x = target_x - x1
    local target_vel_y = target_y - y1

    if dist > max_dist then
        if physcomp or not proj_vel_x then
            PhysicsApplyForce(entity, target_vel_x*100, target_vel_y*100)
        end
        if vel_x then
            vel_y = vel_y - 60
            local target_mag = math.sqrt((target_vel_x^2) + (target_vel_y^2))
            local mag = math.sqrt((vel_x^2) + (vel_y^2)) / 40
            local force_x = ((target_vel_x/target_mag)*mag)
            local force_y = ((target_vel_y/target_mag)*mag)

            vel_x = ((vel_x + force_x)*1.025) + (target_vel_x*dist*0.8)
            vel_y = ((vel_y + force_y)*1.025) + (target_vel_y*dist*0.8)
            EntitySetTransform(entity, target_x, target_y)
            if ComponentGetValue2(chardatacomp, "is_on_ground") == false then vel_x = vel_x*1.16 end
            ComponentSetValue2(chardatacomp, "mVelocity", vel_x, vel_y+60)
        elseif proj_vel_x then
            ComponentSetValue2(projcomp, "mVelocity", proj_vel_x+(target_vel_x*100), proj_vel_y+(target_vel_y*100))
            EntitySetTransform(entity, target_x, target_y)
        end
    end
end

--GET THE INDEX OF A TABLE ITEM:
function get_index(list, x)
	for index, v in pairs(list) do
		if v == x then return index end
	end
	return nil
end--> index of the item, or nil

--RANDOMIZE THE ORDER OF ALL THE TABLE ITEMS WITHIN A TABLE:
function shuffle(t)
    local tbl = {}
    for i = 1, #t do
        tbl[i] = t[i]
    end
    for i = #tbl, 2, -1 do
        local j = math.random(i)
        tbl[i], tbl[j] = tbl[j], tbl[i]
    end
    return tbl
end--> shuffled table

--GET A LIST OF ALL INGESTED MATERIALS:
function IngestedMaterialsList(entity)
	local entity = entity or GetUpdatedEntityID()
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")

	local list = {}
	for k,v in pairs(count_per_material_type) do
		if v ~= 0 then --material exists in the inventory
            material_name = CellFactory_GetName(k-1)
            material_amount = v
            if material_amount > 0 then
				table.insert(list, material_name)
            end
		end
	end   
	return list
end--> table of materials

--GET THE AMOUNT OF A MATERIAL WITHIN AN ENTITY:
function getMaterialCount(material, entity) -- Accepts single material name, array{} of material names, "all", "liquids", "gases", and "solids"(includes solids, sands and statics).
	local entity = entity or GetUpdatedEntityID()
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	if material_inventory == nil then return end
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type");

	if material ~= "all" and material ~= "solids" and material ~= "liquids" and material ~= "gases" and material ~= "fires" and type(material) ~= "table" then
		return tonumber(count_per_material_type[CellFactory_GetType(material)+1])
	end
	local thing = 0
	if type(material) == "table" then
		for i,mat in pairs(material) do
			thing = thing + tonumber(count_per_material_type[CellFactory_GetType(material)+1])
		end
		return thing
	end

	for k,v in pairs(count_per_material_type) do
		if v ~= 0 then --material exists in the inventory
            material_name = CellFactory_GetName(k-1)
            material_amount = v
			if (material == "all") then
				thing = thing + material_amount
			elseif material == "gases" and get_index(CellFactory_GetAllGases(true, false), material_name) ~= nil then
				thing = thing + material_amount
			elseif material == "fires" and get_index(CellFactory_GetAllFires(true, false), material_name) ~= nil then
				thing = thing + material_amount
			--elseif material == "liquids" and get_index(CellFactory_GetAllLiquids( false, false ), material_name) ~= nil then
			--	thing = thing + material_amount
			--elseif material == "solids" and get_index(CellFactory_GetAllSolids( false, false ), material_name) ~= nil then
			--	thing = thing + material_amount
			end
		end
	end   
	return thing
end--> material count

--GIVE WERM:
function add_worms(entity, how_many_of_werm)
    if not EntityGetFirstComponent(entity, "LuaComponent", "DAS_WERMS") then
        if IsPlayer(entity) then
            --script for player worm AI:
            EntityAddComponent( entity, "LuaComponent", {
		    	_tags="DAS_WERMS",
		    	execute_every_n_frame="0",
		    	script_source_file="mods/ir/files/worm.lua",
		    	remove_after_executed="0",
		    	})
        else
            --script for enemy worm AI:
            EntityAddComponent( entity, "LuaComponent", {
		    	_tags="DAS_WERMS",
		    	execute_every_n_frame="0",
		    	script_source_file="mods/ir/files/worm_enemy.lua",
		    	remove_after_executed="0",
		    	})
        end
        EntityAddComponent( entity, "VariableStorageComponent", {_tags="worm_hp",value_float="50",})
        local dmc = EntityGetFirstComponentIncludingDisabled(entity, "DamageModelComponent")
		ComponentSetValue2(dmc, "ragdoll_material", "meat_infested")
    end 
    local x, y = EntityGetTransform(entity)
    for i=1,how_many_of_werm do
        local worm = EntityLoad("mods/ir/files/tapewoim/verlet_woim.xml", x, y)
        local verlet_physics_component = EntityGetFirstComponent(worm, "VerletPhysicsComponent")
        local positions = ComponentGetValue2(verlet_physics_component, "positions")
        local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")
        for i=1,19,2 do
            positions[i] = x
            positions[i+1] = y
            positions_prev[i] = x
            positions_prev[i+1] = y
        end
        ComponentSetValue2( verlet_physics_component, "positions", positions)
        ComponentSetValue2( verlet_physics_component, "positions_prev", positions_prev)
        EntityAddChild(entity, worm)
    end
end

--DESTROY WERM:
function remove_worms(entity, how_many_of_werm)
    local worms_removed = 0
    for i, child in pairs(EntityGetAllChildren(entity)) do
        if EntityGetName(child) == "worm" then
            EntityRemoveFromParent(child)
            EntityAddComponent( child, "SimplePhysicsComponent", {})
            EntityAddComponent( child, "VelocityComponent", {})
            worms_removed = worms_removed + 1
            if worms_removed == how_many_of_werm then return end
        end
    end
    EntityRemoveComponent(entity, EntityGetFirstComponent(entity, "LuaComponent", "DAS_WERMS"))
    EntityRemoveComponent(entity, EntityGetFirstComponent(entity, "VariableStorageComponent", "worm_hp"))
    EntityRemoveIngestionStatusEffect(entity, "WORM")
    EntityRemoveIngestionStatusEffect(entity, "WORM_BERSERK")
    EntityRemoveIngestionStatusEffect(entity, "WORM_CHARM")
end

--GET THE FIRST PROJECTILE THAT'S GOING TO HIT US ON ITS CURRENT TRAJECTORY:
function GetIncomingProjectile(x, y, radius, projectiles_to_ignore)
    local projectiles_to_ignore = projectiles_to_ignore or {}
    local projectiles = EntityGetInRadiusWithTag(x, y, radius, "projectile")
    for i, projectile in ipairs(projectiles) do
        local proj_comp = EntityGetFirstComponent(projectile, "VelocityComponent")
        local px, py = EntityGetTransform(projectile)
        if px and not get_index(projectiles_to_ignore, projectile) then
            local currentdist = math.sqrt(((px-x)^2) + ((py-(y-4))^2))
            local vel_px, vel_py = ComponentGetValue2(proj_comp, "mVelocity")
            local vel_px = vel_px
            local vel_py = vel_py
            local mag = math.sqrt(((vel_px)^2) + ((vel_py)^2))
            local future_x = px + ((vel_px/mag)*currentdist)
            local future_y = py + ((vel_py/mag)*currentdist)
            local futuredist = math.sqrt(((x-future_x)^2) + (((y-4)-future_y)^2))
            local projectile_view_obstructed = RaytraceSurfaces(x,y-4,px,py)
            if futuredist < 10 and futuredist < currentdist and not projectile_view_obstructed then 
                return projectile
            end
        end
    end
end--> projectile_id

--FIND A VISIBLE ENEMY WITHIN RANGE AND LINE OF SIGHT:
function GetAttackableEnemy(x,y,radius,genome,entities_to_ignore)
    local entities_to_ignore = entities_to_ignore or {}
    local newtargs = EntityGetInRadiusWithTag(x, y, radius, "mortal")
    for i, newtarg in ipairs(newtargs) do
        local x2, y2 = EntityGetTransform(newtarg)
        local target_view_obstructed = RaytracePlatforms(x,y,x2,y2)
        local targ_genome = ComponentGetValue2(EntityGetFirstComponent(newtarg, "GenomeDataComponent"), "herd_id")
        if targ_genome and genome ~= targ_genome and not target_view_obstructed and not get_index(entities_to_ignore, newtarg) then
            return newtarg
        end
    end
end--> entity_id

