-----------------------------------------------------------------------------------------
--SETUP
-----------------------------------------------------------------------------------------
dofile_once("mods/ir/files/functions.lua")

local player = EntityGetWithTag("player_unit")[1]
local x, y = EntityGetTransform(player)
local player_chardatacomp = EntityGetFirstComponent(player, "CharacterDataComponent")
local sedc = EntityGetFirstComponent(player, "StatusEffectDataComponent")
local player_platcomp = EntityGetFirstComponent(player, "CharacterPlatformingComponent")
local controlscomp = EntityGetFirstComponent(player, "ControlsComponent")
local worm_hp_comp = EntityGetFirstComponent(player, "VariableStorageComponent", "worm_hp")

local worm_hp = ComponentGetValue2(worm_hp_comp, "value_float")
local pvel_x, pvel_y = ComponentGetValue2(player_chardatacomp, "mVelocity")
local ingest_effects = ComponentGetValue2(sedc, "ingestion_effects")
local berserk = ingest_effects[15]
local charm = ingest_effects[16] 
if EntityHasTag(player, "worm_charmer") then charm = 1 end

EntityRemoveIngestionStatusEffect(player, "WORM")
if math.floor(worm_hp) > 0 then
    EntityIngestMaterial(player, CellFactory_GetType("effect_worms"), math.floor(worm_hp*100))
    AddMaterialInventoryMaterial(player, "effect_worms", getMaterialCount("effect_worms", player) - math.floor(worm_hp*100))
    ComponentSetValue2(worm_hp_comp, "value_float", worm_hp - (1/60))
else
    EntityIngestMaterial(player, CellFactory_GetType("effect_worms"), 1)
    AddMaterialInventoryMaterial(player, "effect_worms", getMaterialCount("effect_worms", player) - 1)
end


local desired_vel_x = 0
local desired_vel_y = -3.888
if ComponentGetValue2(controlscomp, "mButtonDownUp") then
    desired_vel_y = desired_vel_y - 20
elseif ComponentGetValue2(controlscomp, "mButtonDownDown") then
    desired_vel_y = desired_vel_y + 20
end
if ComponentGetValue2(controlscomp, "mButtonDownLeft") then
    desired_vel_x = desired_vel_x - 20
elseif ComponentGetValue2(controlscomp, "mButtonDownRight") then
    desired_vel_x = desired_vel_x + 20
end
local fx, fy = x+(pvel_x/10), y+(pvel_y/10)

-----------------------------------------------------------------------------------------
--LOGIC
-----------------------------------------------------------------------------------------



--Check if the nearest projectile is headed our way, and will it hit something else first:
local nearest_projectile = EntityGetClosestWithTag(x, y-4, "projectile")
local proj_comp = EntityGetFirstComponent(nearest_projectile, "VelocityComponent")
local px, py = EntityGetTransform(nearest_projectile)
local threat = nil
local threat_dist = nil
local threat_future_dist = nil
if px then
    local currentdist = math.sqrt(((px-x)^2) + ((py-(y-4))^2))
    if currentdist < 70 then
        local vel_px, vel_py = ComponentGetValue2(proj_comp, "mVelocity")
        local vel_px = vel_px
        local vel_py = vel_py
        local mag = math.sqrt(((vel_px)^2) + ((vel_py)^2))
        local future_x = px + ((vel_px/mag)*currentdist)
        local future_y = py + ((vel_py/mag)*currentdist)
        local futuredist = math.sqrt(((x-future_x)^2) + (((y-4)-future_y)^2))
        local projectile_view_obstructed = RaytracePlatforms(x,y-4,px,py)
        if futuredist < 10 and not projectile_view_obstructed then 
            threat = nearest_projectile 
            threat_dist = currentdist
            threat_future_dist = futuredist
        end
    end
end

local all_worms = {}
local target_entities = {}
local anchors={}
local strength = 0.0
local grab = false
local attached_one = false

--GET HOW MANY OF WERM:
for i, child in pairs(EntityGetAllChildren( player )) do
    if EntityGetName(child) == "worm" then
        local targ_entity_comp = EntityGetFirstComponent(child, "VariableStorageComponent", "target_entity")
        local targ_entity = ComponentGetValue2(targ_entity_comp, "value_int")
        local state_comp = EntityGetFirstComponent(child, "VariableStorageComponent", "states")
        local targ_x_comp, targ_y_comp = EntityGetFirstComponent(child, "VariableStorageComponent", "target_x"), EntityGetFirstComponent(child, "VariableStorageComponent", "target_y")
        local targ_x, targ_y = ComponentGetValue2(targ_x_comp, "value_int"), ComponentGetValue2(targ_y_comp, "value_int")
        local states = ComponentGetValue2(state_comp, "value_string")
        local coords = {targ_x, targ_y}
        if states == "prosthesisattached" then table.insert(anchors, coords) end
        if string.match(states,"grab") then grab = true end
        table.insert(target_entities, targ_entity)
        table.insert(all_worms, child)
        
    end
end
all_worms = shuffle(all_worms)
local worm_count = #all_worms
local anchor_count = #anchors

--GET FORCE REQUIRED TO MAINTAIN DESIRED PLAYER VELOCITY:
local corrective_x = 0
local corrective_y = 0
if anchor_count > 0 then
    --if pvel_y > 0 and desired_vel_y == 0 then desired_vel_y = -6 end
    corrective_x = (((-pvel_x/10) + desired_vel_x) / anchor_count)
    corrective_y = (((-pvel_y/10) + desired_vel_y) / anchor_count)
end
local corrective_mag = Distance(x, y, x+corrective_x, y+corrective_y)

--FIND WHERE SUPPORT IS NEEDED MOST IN ORDER TO CREATE SAID FORCE:
local x2, y2 = MoveCoordsAlongVector(x, y, x+corrective_x, y+corrective_y, 100) 
local did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x2, y2)
if not did_hit then
    x2, y2 = MoveCoordsAlongVector(x, y, x-corrective_x, y-corrective_y, 100) 
    did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x2, y2)
end
if hit_x == x then did_hit = false end

--CHECK IF PLAYER IS APTLY SUPPORTED ALREADY:
local lacking_support = true
for i, anchor in ipairs(anchors) do
    local ax, ay = anchor[1], anchor[2]
    if Distance(hit_x, hit_y, ax, ay) < ((corrective_mag*10)+5) or not did_hit then
        lacking_support = false
        break
    end
end


--ITERATE ALL THE WERM:
for i, worm in pairs(all_worms) do
    local verlet_physics_component = EntityGetFirstComponent(worm, "VerletPhysicsComponent")
    local state_comp = EntityGetFirstComponent(worm, "VariableStorageComponent", "states")
    local time_comp = EntityGetFirstComponent(worm, "VariableStorageComponent", "time")
    local targ_x_comp, targ_y_comp = EntityGetFirstComponent(worm, "VariableStorageComponent", "target_x"), EntityGetFirstComponent(worm, "VariableStorageComponent", "target_y")
    local targ_entity_comp = EntityGetFirstComponent(worm, "VariableStorageComponent", "target_entity")
    local controls = EntityGetFirstComponentIncludingDisabled(player, "ControlsComponent")

    local mx, my = ComponentGetValue2(controls, "mMousePosition")
    local mousedist = Distance(x, y, mx, my)
    local positions = ComponentGetValue2(verlet_physics_component, "positions")
    local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")
    local endx, endy = 19, 20
    local states = ComponentGetValue2(state_comp, "value_string")
    local time = ComponentGetValue2(time_comp, "value_int")
    local target_entity = ComponentGetValue2(targ_entity_comp, "value_int")
    local targ_x, targ_y = ComponentGetValue2(targ_x_comp, "value_int"), ComponentGetValue2(targ_y_comp, "value_int")
    local cosmetic_force_x = 0
    local cosmetic_force_y = 0
    --WORM DIE IF TOO HUNGER (sad):
    if GameGetFrameNum() % 300 == 0 and math.floor(worm_hp) == 0 and i == 1 then 
        if worm_count == 1 then 
            EntityRemoveComponent(player, EntityGetFirstComponent(player, "LuaComponent", "DAS_WERMS"))
            EntityRemoveComponent(player, EntityGetFirstComponent(player, "VariableStorageComponent", "worm_hp"))
            EntityRemoveIngestionStatusEffect(player, "WORM")
            worm_count = 0
            GamePrint("All of your guests have died of neglect")
        else
            worm_count = worm_count - 1
            GamePrint("1 of your guests has starved to death")
        end
        EntityRemoveFromParent(worm)
        EntityAddComponent( worm, "SimplePhysicsComponent", {})
        EntityAddComponent( worm, "VelocityComponent", {})
    end
    -- :(

    --ACTIVATE GRAB-MODE WHEN KICK:
    if ComponentGetValue2(controls, "mButtonDownKick") == true and string.match(states,"grab") == nil and grab == false and charm > 0 then
        states = states .. "grab"
        ComponentSetValue2(targ_entity_comp, "value_int", 0) 
        target_entity = 0
        local selected_worm = all_worms[math.random(#all_worms)]
    end

    --REACH FOR CURSOR:
    if string.match(states,"grab") then 
        grab = true
        states = states:gsub("KILL", "")

        --REPLACE WERM WITH LONGER ONE:
        if not EntityHasTag(worm, "long") and Distance(mx, my, x, y) > 40 then
            --make copies of current variables:
            local positions2 = positions
            local positions_prev2 = positions_prev

            EntityKill(worm)
            worm = EntityLoad("mods/ir/files/tapewoim/verlet_woim_long.xml", x, y)
            local verlet_physics_component = EntityGetFirstComponent(worm, "VerletPhysicsComponent")
            local positions = ComponentGetValue2(verlet_physics_component, "positions")
            local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")
            for i=1,39,2 do
                if i < 21 then
                    positions[i] = x
                    positions[i+1] = y
                    positions_prev[i] = x
                    positions_prev[i+1] = y
                else
                    positions[i] = positions2[i-20]
                    positions[i+1] = positions2[i-19]
                    positions_prev[i] = positions_prev2[i-20]
                    positions_prev[i+1] = positions_prev2[i-19]
                end
            end
            ComponentSetValue2( verlet_physics_component, "positions", positions)
            ComponentSetValue2( verlet_physics_component, "positions_prev", positions_prev)
            ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "states"), "value_string", states)
            ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_x"), "value_int", targ_x)
            ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_y"), "value_int", targ_y)
            ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_entity"), "value_int", target_entity)
            EntityAddChild(player, worm)
        end
        if EntityHasTag(worm, "long") then endx, endy = 39, 40 end

        --move toward cursor:
        local dist = Distance(mx, my, positions[endx], positions[endy])
        positions_prev[endx] = positions_prev[endx] - ((mx - positions[endx])/dist)
        positions_prev[endy] = positions_prev[endy] - ((my - positions[endy])/dist)
        --attach to stuff:
        if mousedist > 100 then mx, my = MoveCoordsAlongVector(x, y, mx, my, 100) end
        if target_entity == 0 then 
            local nearby_entity = EntityGetInRadius(positions[endx], positions[endy], 10)[1]
            if nearby_entity then  
                local x1, y1 = EntityGetTransform(nearby_entity)
                local root = EntityGetRootEntity(nearby_entity)
                if root ~= player and not EntityHasTag(root, "vegetation") and x1 then
                    if math.sqrt(((mx-x1)^2) + ((my-y1)^2)) < 20 then
                        --LoadGameEffectEntityTo( root, "data/entities/misc/effect_ragdoll_entitify.xml" )
                        ComponentSetValue2(targ_entity_comp, "value_int", root) 
                        target_entity = root
                        table.insert(target_entities, target_entity)
                    end
                end
            end
        end
        --drag said stuff toward cursor:
        local x2, y2 = EntityGetTransform(target_entity)
        if x2 then
            positions[endx] = x2
            positions[endy] = y2
            local target_dist = Distance(x,y,x2,y2)
            EntityApplyForce(target_entity, mx-x2, my-y2)
            cosmetic_force_x = cosmetic_force_x - (mx-x2)
            cosmetic_force_y = cosmetic_force_y - (my-y2)
            local dmc = EntityGetFirstComponent(target_entity, "DamageModelComponent")
            if dmc then
                local hp = ComponentGetValue2(dmc, "hp")
                if berserk > 0 then
                    EntityInflictDamage(target_entity, 0.04, "DAMAGE_MATERIAL", "blood donation", "NONE", 0, 0, player)
                    local blood = ComponentGetValue2(dmc, "blood_spray_material")
                    if blood then EntityIngestMaterial(player, CellFactory_GetType(blood), 10) end
                    hp = hp - 0.04
                end
                if hp >= 0 then
                    LoadGameEffectEntityTo( target_entity, "data/entities/misc/effect_ragdoll_entitify_short.xml" )
                end
            end
        end
        --let go:
        if ComponentGetValue2(controls, "mButtonDownKick") == false or charm == 0 then 
            states = states:gsub("grab", "")
            ComponentSetValue2(targ_entity_comp, "value_int", 0) 
            target_entity = 0
        elseif (not x2) and target_entity ~= 0 then
            local nearby_ragdoll = EntityGetInRadiusWithTag(positions[endx], positions[endy], 10, "ragdoll_entity")[1]
            if not x2 and target_entity ~= 0 and nearby_ragdoll then
                target_entity = nearby_ragdoll
                ComponentSetValue2(targ_entity_comp, "value_int", nearby_ragdoll) 
                table.insert(target_entities, target_entity)
            else
                ComponentSetValue2(targ_entity_comp, "value_int", 0) 
                target_entity = 0
            end
        end
    end

    --REPLACE WERM WITH SHORTER ONE:
    if EntityHasTag(worm, "long") and not string.match(states,"grab") then
        --make copies of current variables:
        local positions2 = positions
        local positions_prev2 = positions_prev

        EntityKill(worm)
        worm = EntityLoad("mods/ir/files/tapewoim/verlet_woim.xml", x, y)
        local verlet_physics_component = EntityGetFirstComponent(worm, "VerletPhysicsComponent")
        local positions = ComponentGetValue2(verlet_physics_component, "positions")
        local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")
        for i=1,19,2 do
            positions[i] = positions2[i+20]
            positions[i+1] = positions2[i+21]
            positions_prev[i] = positions_prev2[i+20]
            positions_prev[i+1] = positions_prev2[i+21]
        end
        ComponentSetValue2( verlet_physics_component, "positions", positions)
        ComponentSetValue2( verlet_physics_component, "positions_prev", positions_prev)
        ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "states"), "value_string", states)
        ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_x"), "value_int", targ_x)
        ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_y"), "value_int", targ_y)
        ComponentSetValue2(EntityGetFirstComponent(worm, "VariableStorageComponent", "target_entity"), "value_int", target_entity)
        EntityAddChild(player, worm)
        endx, endy = 19, 20
    end

    --GET TARGET ENTITY OR FIND A NEW ONE:
    local x2, y2 = EntityGetTransform(target_entity)
    if not x2 and not string.match(states,"grab") and not string.match(states,"KILL") and berserk > 0 then
        --Target incoming projectiles, if any:
        if threat and not get_index(target_entities, threat) then 
            ComponentSetValue2(targ_entity_comp, "value_int", threat) 
            states = states .. "KILL"
            target_entity = threat
            table.insert(target_entities, target_entity)
            time = math.random(80)
            GamePlaySound( "data/audio/Desktop/player.bank", "player/step_slime", x, y )
        else
            --If none exist, find a living victim:
            local newtargs = EntityGetInRadiusWithTag(x, y, 60, "homing_target")
            for i, newtarg in ipairs(newtargs) do
                local x2, y2 = EntityGetTransform(newtarg)
                local target_view_obstructed = RaytracePlatforms(x,y,x2,y2)
                if EntityGetRootEntity(newtarg) ~= player and not target_view_obstructed and not get_index(target_entities, newtarg) and not EntityHasTag(target_entity, "prop") then
                    ComponentSetValue2(targ_entity_comp, "value_int", newtarg) 
                    states = states .. "KILL"
                    target_entity = newtarg
                    table.insert(target_entities, target_entity)
                    LoadGameEffectEntityTo( target_entity, "data/entities/misc/effect_ragdoll_entitify.xml" )
                    time = math.random(70)
                    GamePlaySound( "data/audio/Desktop/player.bank", "player/land_slime", x, y )
                    break
                end
            end
        end
        if not EntityGetTransform(target_entity) then
            if string.match(states,"KILL") then states = states:gsub("KILL", "") end
            target_entity = nil
        end
    else
        table.insert(target_entities, target_entity)
    end
    
    --ATTACH TO TARGET ENTITY:
    if string.match(states,"KILL") and not string.match(states,"grab") then
        local x2, y2 = EntityGetTransform(target_entity)
        local nearby_ragdoll = EntityGetInRadiusWithTag(positions[19], positions[20], 10, "ragdoll_entity")[1]
        if (not x2) and nearby_ragdoll then
            target_entity = nearby_ragdoll
            ComponentSetValue2(targ_entity_comp, "value_int", nearby_ragdoll) 
            table.insert(target_entities, target_entity)
            x2, y2 = EntityGetTransform(target_entity)
            time = math.random(70)
        elseif not x2 then
            ComponentSetValue2(targ_entity_comp, "value_int", 0)
            states = states:gsub("KILL", "")
            target_entity = 0
            goto kill_end
        end
        positions[19] = x2
        positions[20] = y2
        positions_prev[19] = x2
        positions_prev[20] = y2
        if not EntityHasTag(target_entity, "projectile") then
            constrain(player, x2, y2, 70)
        else
            local factor = 1
            if target_entity % 2 == 0 then factor = -1 end
            local threat_dist = math.sqrt(((x-x2)^2) + ((y-y2)^2))
            local force_out_x, force_out_y = ((x2-x)/(threat_dist^2)), ((y2-y)/(threat_dist^2))
            local force_parallel_x, force_parallel_y = factor * ((y2-y)/threat_dist), -factor *((x2-x)/threat_dist)
            local forcex, forcey = force_out_x+force_parallel_x, force_out_y+force_parallel_y
            EntityApplyForce(target_entity, forcex*10, forcey*10)
            EntityApplyForce(player, -forcex, -forcey)
            cosmetic_force_x = cosmetic_force_x + (forcex*10)
            cosmetic_force_y = cosmetic_force_y + (forcey*10)
            time = time + 1
            if threat_dist > 70 or time > 100 then
                ComponentSetValue2(targ_entity_comp, "value_int", 0)
                states = states:gsub("KILL", "")
            end
        end
        constrain(target_entity, x, y, 70)
        --if entity killed then throw corpse, else drain hp:
        if EntityHasTag(target_entity, "ragdoll_entity") then
            PhysicsApplyForce(target_entity, (x-x2), (y-y2))
            cosmetic_force_x = cosmetic_force_x + (x-x2)
            cosmetic_force_y = cosmetic_force_y + (y-y2)
            time = time + 1
            if time > 100 then 
                ComponentSetValue2(targ_entity_comp, "value_int", 0)
                states = states:gsub("KILL", "")
            end
        elseif not EntityHasTag(target_entity, "projectile") then
            local dmc = EntityGetFirstComponent(target_entity, "DamageModelComponent")
            local hp = ComponentGetValue2(dmc, "hp")
            local blood = ComponentGetValue2(dmc, "blood_spray_material")
            if blood then EntityIngestMaterial(player, CellFactory_GetType(blood), 10) end
            if hp - 0.04 >= 0 then
                LoadGameEffectEntityTo( target_entity, "data/entities/misc/effect_ragdoll_entitify_short.xml" )
            end
            EntityInflictDamage(target_entity, 0.04, "DAMAGE_MATERIAL", "blood donation", "NONE", 0, 0, player)
        end
        ::kill_end::
    end
    
    --FIND SURFACES TO ATTACH TO:
    if states == "prosthesis" and not attached_one and charm > 0 then
        if not did_hit or not lacking_support then
            x2, y2 = MoveCoordsAlongVector(x+corrective_x, y+corrective_y, x+corrective_x+math.random(-30,30), y+corrective_y+math.random(-30,30), 100) 
            did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x2, y2)
        end
        if hit_x == x then did_hit = false end

        if did_hit then
            states = states .. "attached"
            ComponentSetValue2(targ_x_comp, "value_int", hit_x)
            ComponentSetValue2(targ_y_comp, "value_int", hit_y)
            targ_x = hit_x
            targ_y = hit_y
            attached_one = true
            lacking_support = false
            GamePlaySound( "data/audio/Desktop/player.bank", "player/step_slime", hit_x, hit_y )
        end
        if worm_count < 5 or ComponentGetValue2(player_chardatacomp, "is_on_ground") then states = "" end
    elseif states == "" and worm_count > 4 and not ComponentGetValue2(player_chardatacomp, "is_on_ground") then
        states = "prosthesis"
    end
    --MOVE PLAYER:
    if states == "prosthesisattached" then
        local dist3 = Distance(x,y,targ_x,targ_y)
        local plusx, plusy = MoveCoordsAlongVector(x, y, targ_x, targ_y, dist3*1.1)
        positions[19] = plusx
        positions[20] = plusy
        positions_prev[19] = plusx
        positions_prev[20] = plusy

        local dist = math.sqrt((((x+(corrective_x/10))-targ_x)^2) + (((y+(corrective_y/10))-targ_y)^2))--get the length the worm would be if the player were moving at the desired velocity
        local player_targ_x, player_targ_y = MoveCoordsAlongVector(targ_x, targ_y, x, y, dist) --force from worm to player
        local mag = math.sqrt(((player_targ_x-targ_x)^2) + ((player_targ_y-targ_y)^2)) --magnitude of force
        if not string.match(tostring((player_targ_x-x) * 20),"nan") then 
            pvel_x = pvel_x + (player_targ_x-x) * 20
            pvel_y = pvel_y + (player_targ_y-y) * 20
            cosmetic_force_x = cosmetic_force_x + ((player_targ_x-x) * 20)
            cosmetic_force_y = cosmetic_force_y + ((player_targ_y-y) * 20)
        end
        if Distance(targ_x, targ_y, x+corrective_x, y+corrective_y) > 100 or worm_count < 5 or ComponentGetValue2(player_chardatacomp, "is_on_ground") or charm == 0 then
            states = states:gsub("attached", "")
        elseif lacking_support then
            if not did_hit then
                x2, y2 = MoveCoordsAlongVector(x+corrective_x, y+corrective_y, x+corrective_x+math.random(-30,30), y+corrective_y+math.random(-30,30), 100) 
                did_hit, hit_x, hit_y = RaytracePlatforms(x, y, x2, y2)
            end
            if hit_x == x then did_hit = false end

            if did_hit then
                ComponentSetValue2(targ_x_comp, "value_int", hit_x)
                ComponentSetValue2(targ_y_comp, "value_int", hit_y)
                attached_one = true
                lacking_support = false
                GamePlaySound( "data/audio/Desktop/player.bank", "player/step_slime", hit_x, hit_y )
            end
        end
    end

    --WIGGLE AROUND:
    for i=3,endx,2 do
        positions_prev[i] = ((positions_prev[i]*9) + positions_prev[i-2]) / 10 --inherits some of the next chain-link's x velocity
        positions_prev[i+1] = (((positions_prev[i+1]*9) + positions_prev[i-1]) / 10) + 0.01 --inherits some of the next chain-link's y velocity, plus an upward force
        positions_prev[i] = positions_prev[i] + (cosmetic_force_x/100)
        positions_prev[i+1] = positions_prev[i+1] + (cosmetic_force_y/100)
    end
    --push the end of the chain around in random directions:
    positions_prev[endx] = positions_prev[endx] + math.random(-20,20)/20
    positions_prev[endy] = positions_prev[endy] + math.random(-20,20)/20
    --lay eggs:
    if math.random(1000) == 1 then
		GameCreateParticle("worm_egg", positions[endx], positions[endy], 1, 0, 0, false)
    end
    --APPLY_CHANGES:
    ComponentSetValue2(verlet_physics_component, "positions_prev", positions_prev)
    ComponentSetValue2(verlet_physics_component, "positions", positions)
    ComponentSetValue2(time_comp, "value_int", time)
    ComponentSetValue2(state_comp, "value_string", states)
end
if worm_count > 4 and ComponentGetValue2(player_chardatacomp, "is_on_ground") == false and charm > 0 then
    pvel_x = pvel_x*1.13
    if not EntityHasTag(player, "paralyzed") then
        ComponentSetValue2(player_chardatacomp, "fly_time_max", 0)
        ComponentSetValue2(player_platcomp, "run_velocity", 0)
        ComponentSetValue2(player_platcomp, "fly_velocity_x", 0)
        EntityAddTag(player, "paralyzed")
    end
elseif EntityHasTag(player, "paralyzed") then
    ComponentSetValue2(player_chardatacomp, "fly_time_max", 3)
    ComponentSetValue2(player_platcomp, "run_velocity", 160)
    ComponentSetValue2(player_platcomp, "fly_velocity_x", 52)
    EntityRemoveTag(player, "paralyzed")
end
ComponentSetValue2(player_chardatacomp, "mVelocity", pvel_x, pvel_y)
