-- all functions below are optional and can be left out

--[[

function OnModPreInit()
	print("Mod - OnModPreInit()") -- First this is called for all mods
end

function OnModInit()
	print("Mod - OnModInit()") -- After that this is called for all mods
end

function OnModPostInit()
	print("Mod - OnModPostInit()") -- Then this is called for all mods
end

function OnPlayerSpawned(player) -- This runs when player entity has been created
	local x, y = EntityGetTransform(player)
	CreateItemActionEntity("MY_CUSTOM_SPELL", x+20, y)
	CreateItemActionEntity("MY_CUSTOM_SPELL", x, y)
end

function OnWorldPreUpdate() -- This is called every time the game is about to start updating the world
	GamePrint( "Pre-update hook " .. tostring(GameGetFrameNum()) )
end

function OnWorldInitialized() -- This is called once the game world is initialized. Doesn't ensure any world chunks actually exist. Use OnPlayerSpawned to ensure the chunks around player have been loaded or created.
end
]]--

function OnWorldPostUpdate() -- This is called every time the game has finished updating the world
	local player = EntityGetWithTag("player_unit")[1]
	local x, y = EntityGetTransform(player)
	local entities = EntityGetWithTag("homing_target")
	local entity = entities[math.random(#entities)]

	if not EntityHasTag(entity, "nat_werm") then
		local worm_charmer = EntityHasTag(entity, "worm_charmer")

		--CHECK IF ENTITY'S BLOOD CAN SUPPORT A TAPEWORM INFESTATION
		local dmc = EntityGetFirstComponent(entity, "DamageModelComponent")
		local blood_type = ComponentGetValue2(dmc, "blood_spray_material") 
		if blood_type == "" then blood_type = ComponentGetValue2(dmc, "blood_material") end

		local blood_is_organic = false
		local blood_is_liquid = false
		for i, tag in ipairs(CellFactory_GetTags(CellFactory_GetType(blood_type))) do
			if tag == "[liquid]" then
				blood_is_liquid = true
			end
			if tag == "[hot]" or tag == "[fire]" or tag == "[lava]" or tag == "[burnable]" then 
				blood_is_organic = false
				break
			end
			blood_is_organic = true
		end

		local count = math.random(1,math.random(1,math.random(1,10)))
		if worm_charmer then count = math.random(5,7) end
		if entity and player and entity ~= player and count > 0 and blood_is_liquid and blood_is_organic then
			if math.random(30) == 1 or worm_charmer then
				EntityAddComponent( entity, "LuaComponent", {
					_tags="DAS_WERMS",
					execute_every_n_frame="0",
					script_source_file="mods/ir/files/worm_enemy.lua",
					remove_after_executed="0",
					})
				local dmc = EntityGetFirstComponentIncludingDisabled(entity, "DamageModelComponent")
				ComponentSetValue2(dmc, "ragdoll_material", "meat_infested")
				local x2, y2 = EntityGetTransform(entity)
				for i=1,count do
					local worm = EntityLoad("mods/ir/files/tapewoim/verlet_woim.xml", x2, y2-4)
					local verlet_physics_component = EntityGetFirstComponent(worm, "VerletPhysicsComponent")
					local positions = ComponentGetValue2(verlet_physics_component, "positions")
					local positions_prev = ComponentGetValue2(verlet_physics_component, "positions_prev")
					for i=1,19,2 do
						positions[i] = x2
						positions[i+1] = y2
						positions_prev[i] = x2
						positions_prev[i+1] = y2
					end
					ComponentSetValue2( verlet_physics_component, "positions", positions)
					ComponentSetValue2( verlet_physics_component, "positions_prev", positions_prev)
					EntityAddChild(entity, worm)
				end
			end

			if count > 0 then
				--local displayname = GameTextGetTranslatedOrNot(EntityGetName(entity)) or "creature"
				--GamePrint("added " .. count .. " worms to " .. displayname .. " with " .. blood_type .. " blood")
				if math.random(10) == 1 and not worm_charmer then 
					EntityAddTag(entity, "worm_charmer") 
					--GamePrint(displayname .. " can charm parasites")
				end
			end
		end
	end
	EntityAddTag(entity, "nat_werm")
end



function OnPlayerSpawned(player) -- This runs when player entity has been created
	dofile_once("data/scripts/lib/utilities.lua")
	local x, y = EntityGetTransform(player)
	-----------------------------------------------------------------------------------------
	--SET UP DIGESTIVE SYSTEM
	-----------------------------------------------------------------------------------------
	if not GameHasFlagRun( "internal_reactions_started" ) then
		local x, y = EntityGetTransform(player)
		local can_beans = EntityLoad( "mods/ir/files/physics_can_beans.xml", x, -800 ) 
		GamePickUpInventoryItem( player, can_beans, false )
		

		if ModSettingGet("ir.stomach_type") == "tri" then
			EntityAddComponent( player, "LuaComponent", {
				_tags="DIGESTION",
				execute_every_n_frame="0",
				script_source_file="mods/ir/files/react.lua",
				remove_after_executed="0",
				})
		else
			EntityAddComponent( player, "LuaComponent", {
				_tags="DIGESTION",
				execute_every_n_frame="0",
				script_source_file="mods/ir/react(old).lua",
				remove_after_executed="0",
				})
		end
		
		local bladder = EntityCreateNew()
		local colon = EntityCreateNew()
		EntityAddTag( bladder, "bladder" )
		EntityAddTag( colon, "colon" )
		EntityAddComponent( bladder, "MaterialInventoryComponent", {})
		EntityAddComponent( colon, "MaterialInventoryComponent", {})
		EntityAddChild( player, bladder )
		EntityAddChild( player, colon )

		EntityAddComponent( player, "VariableStorageComponent", {
			_tags="expelletory_status",
			value_int="0",
			})
		EntityAddComponent( player, "VariableStorageComponent", {
			_tags="crouching",
			value_string="false",
			})
		EntityAddComponent( player, "VariableStorageComponent", {
			_tags="fpc",
			value_int="0",
			})
		if ModSettingGet("ir.excretory_type") == "convenient" then
			EntityAddComponent( player, "ParticleEmitterComponent", {
				_tags="URINE",
				_enabled="1",
				emitted_material_name="urine",
				emission_interval_max_frames="0",
				emission_interval_min_frames="0",
				emit_real_particles="1",
				draw_as_long="1",
				render_back="1",
				b2_force="0.6",
				x_pos_offset_max="0",
				x_pos_offset_min="0",
				x_vel_max="-150.1366",
				x_vel_min="-125.4561",
				y_pos_offset_max="0",
				y_pos_offset_min="0",
				y_vel_max="0",
				y_vel_min="0",
				})
		else
			EntityAddComponent( player, "ParticleEmitterComponent", {
				_tags="URINE",
				_enabled="1",
				emitted_material_name="urine",
				emission_interval_max_frames="0",
				emission_interval_min_frames="0",
				emit_real_particles="1",
				create_real_particles="1",
				draw_as_long="1",
				render_back="1",
				b2_force="0.6",
				x_pos_offset_max="0",
				x_pos_offset_min="0",
				x_vel_max="-150.1366",
				x_vel_min="-125.4561",
				y_pos_offset_max="0",
				y_pos_offset_min="0",
				y_vel_max="0",
				y_vel_min="0",
				})
		end
		EntityAddComponent( player, "ParticleEmitterComponent", {
			_tags="FECES",
			_enabled="1",
			emitted_material_name="poo",
			emission_interval_max_frames="0",
			emission_interval_min_frames="0",
			fade_based_on_lifetime="1",
			emit_real_particles="1",
			cosmetic_force_create="1",
			draw_as_long="1",
			render_back="1",
			render_on_grid="1",
			airflow_force="2",
			airflow_time="1",
			airflow_scale="0.1",
			b2_force="0.6",
			x_pos_offset_max="0",
			x_pos_offset_min="0",
			x_vel_max="-10",
			x_vel_min="-10",
			y_pos_offset_max="0",
			y_pos_offset_min="0",
			y_vel_max="0",
			y_vel_min="0",
			})
		EntityAddComponent( player, "ParticleEmitterComponent", {
			_tags="FARTICLES",
			_enabled="1",
			emitted_material_name="acid_gas",
			emission_interval_max_frames="0",
			emission_interval_min_frames="0",
			lifetime_min="0.3",
			lifetime_max="5",
			fade_based_on_lifetime="1",
			emit_real_particles="0",
			create_real_particles="0",
			emit_cosmetic_particles="1",
			cosmetic_force_create="1",
			render_on_grid="1",
			draw_as_long="0",
			render_back="0",
			friction="1",
			airflow_force="2",
			airflow_time="1",
			airflow_scale="0.1",
			b2_force="0.6",
			x_pos_offset_max="0",
			x_pos_offset_min="0",
			y_pos_offset_max="0",
			y_pos_offset_min="0",
			x_vel_max="0",
			x_vel_min="-30",
			y_vel_max="0",
			y_vel_min="0",
			})
		EntityAddComponent( player, "HotspotComponent", {
			_tags="mouth",
			sprite_hotspot_name="cape",
			transform_with_scale="1",
			})
		EntityAddComponent( player, "HotspotComponent", {
			_tags="butt",
			sprite_hotspot_name="belt",
			transform_with_scale="1",
			})
			
		local peecomp = EntityGetFirstComponentIncludingDisabled( player, "ParticleEmitterComponent", "URINE" )
		local poocomp = EntityGetFirstComponentIncludingDisabled( player, "ParticleEmitterComponent", "FECES" )
		local fartcomp = EntityGetFirstComponentIncludingDisabled( player, "ParticleEmitterComponent", "FARTICLES" )
		local ingestioncomp = EntityGetFirstComponentIncludingDisabled( player, "IngestionComponent")
		local platcomp = EntityGetFirstComponentIncludingDisabled( entity, "CharacterPlatformingComponent" )
		ComponentSetValue2( platcomp, "run_velocity", 160)
		ComponentSetValue2( platcomp, "fly_velocity_x", 58)
		ComponentSetValue2(peecomp, "offset", 1, 0)
		ComponentSetValue2(poocomp, "offset", -5, 0)
		ComponentSetValue2(fartcomp, "offset", -3, 0)
		ComponentSetValue2(fartcomp, "gravity", 0, -60)
		ComponentSetValue2(ingestioncomp, "ingestion_satiation_material_tag", "none")
		GameAddFlagRun( "internal_reactions_started" )
	end
end

function OnMagicNumbersAndWorldSeedInitialized() -- this is the last point where the Mod* API is available. after this materials.xml will be loaded.
	local x = ProceduralRandom(0,0)
	print( "===================================== random " .. tostring(x) )
end

-- This code runs when all mods' filesystems are registered
ModLuaFileAppend("data/scripts/items/potion.lua", "mods/ir/files/make_beans.lua")
ModLuaFileAppend("data/scripts/perks/perk_list.lua", "mods/ir/files/perk_changes.lua")
ModLuaFileAppend("data/scripts/status_effects/status_list.lua", "mods/ir/files/status_list_appends.lua")
ModLuaFileAppend("data/scripts/biomes/fungicave.lua", "mods/ir/files/spawn_wormwizard.lua")
ModLuaFileAppend("data/scripts/biomes/wizardcave.lua", "mods/ir/files/spawn_wormwizard.lua")
ModLuaFileAppend("data/scripts/biomes/rainforest.lua", "mods/ir/files/spawn_wormwizard.lua")
ModLuaFileAppend("data/scripts/biomes/the_end.lua", "mods/ir/files/spawn_wormwizard.lua")

local xml2lua = dofile("mods/ir/lib/xml2lua/xml2lua.lua")
local handler = dofile("mods/ir/lib/xml2lua/xmlhandler/tree.lua")
local parser = xml2lua.parser(handler)
local materials = ModTextFileGetContent("data/materials.xml")
parser:parse(materials)
for i, p in pairs(handler.root.Materials) do
	if i == "CellDataChild" or i == "CellData" then
		for i2, p2 in pairs(handler.root.Materials[i]) do
			if p2.StatusEffects ~= nil then
				if p2.StatusEffects.Ingestion ~= nil then
					if p2.StatusEffects.Ingestion.StatusEffect ~= nil then
						if p2.StatusEffects.Ingestion.StatusEffect._attr ~= nil then
							if p2.StatusEffects.Ingestion.StatusEffect._attr.type ~= nil then --This is so stupid, why does Lua have zero try-except functionality for indexing a possibly-nil value? What a horrendous jumble of "if" statements I've been forced to make, just to find out if a table value exists. Lua sucks
								local ingestion_effect = p2.StatusEffects.Ingestion.StatusEffect._attr.type
								local material_name = p2._attr.name
								if material_name == "meat_done" then
									p2._attr.ui_name = "Cooked Meat"
									p2.StatusEffects.Ingestion.StatusEffect._attr.type = "NONE"
								elseif material_name == "slime" or material_name == "oil" then
									p2.StatusEffects.Ingestion.StatusEffect._attr.type = "FOOD_POISONING"
								elseif material_name == "meat_warm" or material_name == "meat_burned" or material_name == "meat_hot" then
									print("removing material " .. material_name)
									handler.root.Materials[i][i2] = nil
								end
							end
						end
					end
				end
			end
		end
	elseif i == "Reaction" then
		for i2, p2 in pairs(handler.root.Materials[i]) do
			if(p2._attr ~= nil)then
				local ip1 = p2._attr.input_cell1
				local ip2 = p2._attr.input_cell2
				local op1 = p2._attr.output_cell1
				local op2 = p2._attr.output_cell2
				if op1 == "meat_done" then
					p2._attr.probability = 100
					p2._attr.input_cell1 = "meat_done"
					p2._attr.output_cell1 = "meat_deep_fried"
				elseif ip1 == "meat_warm" or op1 == "meat_warm" or  ip1 == "meat_hot" or op1 == "meat_hot" or op1 == "meat_burned" then
					print("removing reaction " .. ip1 .. " + " .. ip2 .. " --> " .. op1 .. " + " .. op2)
					handler.root.Materials[i][i2] = nil
				end
			end
		end
	end
end

ModTextFileSetContent("data/materials.xml", xml2lua.toXml(handler.root.Materials, "Materials"))
ModMaterialsFileAdd( "mods/ir/files/materials_poo.xml" ) 

if ModSettingGet("ir.flowy_gas") == "yes" then
	ModMaterialsFileAdd( "mods/ir/files/materials_gas.xml" )
end
if ModSettingGet("ir.difficulty") ~= "dumb" then
	ModTextFileSetContent("data/entities/items/pickup/heart_fullhp_temple.xml", ModTextFileGetContent("mods/ir/files/heart_fullhp_temple.xml"))
end