-----------------------------------------------------------------------------------------
--SETUP
-----------------------------------------------------------------------------------------
dofile_once( "data/scripts/game_helpers.lua" )
dofile_once("data/scripts/lib/utilities.lua")

local entity = GetUpdatedEntityID()
x,y = EntityGetTransform(entity)

local expelletory_status = EntityGetFirstComponentIncludingDisabled(entity, "VariableStorageComponent", "expelletory_status")

local vomit_entity = nil
local vomitcomp = nil
for i, child in pairs(EntityGetAllChildren( entity )) do
	if EntityHasTag(child, "vomit") then
		vomit_entity = child
		vomitcomp = EntityGetFirstComponentIncludingDisabled( vomit_entity, "ParticleEmitterComponent", "VOMIT" )
	end
end
if vomit_entity == nil then 
	vomit_entity = EntityLoad( "data/entities/particles/vomit.xml", x, y )
	EntityAddTag( vomit_entity, "vomit" )
	EntityAddChild( entity, vomit_entity )
	vomitcomp = EntityGetFirstComponentIncludingDisabled( vomit_entity, "ParticleEmitterComponent", "VOMIT" )
end

local peecomp = EntityGetFirstComponentIncludingDisabled( entity, "ParticleEmitterComponent", "URINE" )
local poocomp = EntityGetFirstComponentIncludingDisabled( entity, "ParticleEmitterComponent", "FECES" )
local fartcomp = EntityGetFirstComponentIncludingDisabled( entity, "ParticleEmitterComponent", "FARTICLES" )

local controls = EntityGetFirstComponentIncludingDisabled( entity, "ControlsComponent" )
local dmc = EntityGetFirstComponentIncludingDisabled(entity, "DamageModelComponent")
local stats = EntityGetFirstComponentIncludingDisabled(entity, "StatusEffectDataComponent")
local effects = ComponentGetValue2(stats, "stain_effects")
local hp = ComponentGetValue2(dmc, "hp")
local max_hp = ComponentGetValue2(dmc, "max_hp")
local entity_vel_x, entity_vel_y = GameGetVelocityCompVelocity(entity)
local crouching = EntityGetFirstComponentIncludingDisabled( entity, "VariableStorageComponent", "crouching" )
crouch = false

-----------------------------------------------------------------------------------------
--FUNCTIONS
-----------------------------------------------------------------------------------------
function SkipFrames(number)
	if GameGetFrameNum() % number == 0 then return true end
	return false
end

function shuffle(t)
  local tbl = {}
  for i = 1, #t do
    tbl[i] = t[i]
  end
  for i = #tbl, 2, -1 do
    local j = math.random(i)
    tbl[i], tbl[j] = tbl[j], tbl[i]
  end
  return tbl
end
function get_index(list, x)
	for index, v in pairs(list) do
		if v == x then return index end
	end
	return nil
end
function IngestedMaterialsList()
  	local entity = GetUpdatedEntityID()
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")

	if material_inventory == nil then return end

	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")
	local list = {}
	for k,v in pairs(count_per_material_type) do
		if v ~= 0 then --material exists in the inventory
                   		material_name = CellFactory_GetName(k-1)
                    		material_amount = v

                    		if material_amount > 0 then
				table.insert(list, material_name)
                    		end
		end
	end   
	return list
end
function getMaterialCount(material) -- Accepts single material name, array{} of material names, "all", "liquids", "gases", "fires", and "solids"(includes solids, sands and statics).
	entity = GetUpdatedEntityID()
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	if material_inventory == nil then return end
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type");

	if material ~= "all" and material ~= "solids" and material ~= "liquids" and material ~= "gases" and material ~= "fires" and type(material) ~= "table" then
		return tonumber(count_per_material_type[CellFactory_GetType(material)+1])
	end
	local thing = 0
	if type(material) == "table" then
		for i,mat in pairs(material) do
			thing = thing + tonumber(count_per_material_type[CellFactory_GetType(mat)+1])
		end
		return thing
	end

	for k,v in pairs(count_per_material_type) do
		if v ~= 0 then --material exists in the inventory
                   		material_name = CellFactory_GetName(k-1)
                    		material_amount = v
			if (material == "all") then
				thing = thing + material_amount
			elseif material == "solids" and get_index(CellFactory_GetAllLiquids( false, false ), material_name) == nil and get_index(CellFactory_GetAllGases(true, false), material_name) == nil then
				thing = thing + material_amount
			elseif material == "liquids" and get_index(CellFactory_GetAllLiquids( false, false ), material_name) ~= nil then
				thing = thing + material_amount
			elseif material == "gases" and get_index(CellFactory_GetAllGases(true, false), material_name) ~= nil then
				thing = thing + material_amount
			elseif material == "fires" and get_index(CellFactory_GetAllFires(true, false), material_name) ~= nil then
				thing = thing + material_amount
			end
		end
	end   
	return thing
end




-----------------------------------------------------------------------------------------
--INTERNAL REACTIONS
-----------------------------------------------------------------------------------------
local sick = "FOOD_POISONING"
local runs = "DIARRHEA"



					
local reactable =	{"blood",		"water",		"water_swamp",	"water_salt",	"oil",		"radioactive_liquid",	"poison",		"mud",		"meat",		"meat_warm",	"meat_hot",	"meat_done",	"meat_marinated",	"honey",		"meat_helpless",	"meat_helpless_done",	"meat_helpless_marinated",	"grass",		"grass_ice",	"grass_dry",	"moss",		"rotten_meat",	"rotten_meat_radioactive",	"soil",		"meat_slime",	"meat_slime_sand",		"slime",		"beans",		"dry_poo",		"fire",		"alcohol"}
local healthiness = 	{0.002,		0,		-0.001,		-0.001,		0,		-0.01,		-0.001,		-0.001,		0.003,		0.003,		0.003,		0.009,		0.02,		0.02,		0.02,		0.04,			0.06,			0.001,		0.001,		0.001,		0.001,		0.003,		-0.01,			0,		0.003,		0.003,			-0.004,		0.1,		-0.001,		-0.01,		0.002}
local digests_to = 	{"urine",		"urine",		"diarrhea",		"vomit",		"vomit",		"urine",		"poison",		"diarrhea",		"poo",		"poo",		"poo",		"poo",		"poo",		"diarrhea",		"poo",		"poo",			"poo",			"poo",		"poo",		"poo",		"poo",		"poo",		"poo",			"poo",		"diarrhea",		"diarrhea",			"vomit",		"poo",		"poo",		"smoke",		"urine"}


EntityRemoveIngestionStatusEffect( entity, "IRON_STOMACH" )
local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")
local ambrosiacount = count_per_material_type[CellFactory_GetType("magic_liquid_protection_all")+1]
local vomitcount = count_per_material_type[CellFactory_GetType("vomit")+1]
if ambrosiacount > 0 then
	local extra = 0
	if SkipFrames(10) then extra = 1 end
	if vomitcount < 1 and ComponentGetValue2(expelletory_status, "value_int") ~= 5 then
		EntityIngestMaterial( entity, CellFactory_GetType("magic_liquid_protection_all"), ambrosiacount ) 
		AddMaterialInventoryMaterial(entity, "magic_liquid_protection_all", ambrosiacount - extra)
		AddMaterialInventoryMaterial(entity, "urine", count_per_material_type[CellFactory_GetType("urine")+1] + extra)
	else
		AddMaterialInventoryMaterial(entity, "magic_liquid_protection_all", ambrosiacount - extra)
		AddMaterialInventoryMaterial(entity, "vomit", vomitcount + extra)
	end
end

--INTERMATERIAL
dofile("mods/ir/files/stringstore/stringstore.lua")
dofile("mods/ir/files/stringstore/noitaglobalstore.lua")
MY_EXTERN_GLOBAL = stringstore.open_store(stringstore.noita.global("MY_EXTERN_GLOBAL"))



if MY_EXTERN_GLOBAL.subtable == nil then
	function parse_tags(tags)
		local output = {}
		if tags == nil then
			return output
		end
		local a = string.gmatch(tags, "([^,]+)")
		for i in a do
			i = i:gsub("%[", "")
			i = i:gsub("%]", "")
			output[i] = true
		end
		return output
	end
	existing_reactions = {}
	material_tags = {}
	material_translations = {}
	local xml2lua = dofile("mods/ir/lib/xml2lua/xml2lua.lua")
	local handler = dofile("mods/ir/lib/xml2lua/xmlhandler/tree.lua")
	local parser = xml2lua.parser(handler)
	local materials = ModTextFileGetContent("data/materials.xml")
	parser:parse(materials)
	for i, p in pairs(handler.root.Materials) do
		if i == "CellDataChild" or i == "CellData" then
			for i2, p2 in pairs(handler.root.Materials[i]) do
				if(p2._attr ~= nil)then
					if(p2._attr.tags ~= nil)then
						local tags = parse_tags(p2._attr.tags)
						local name = p2._attr.name
						local ui_name = p2._attr.ui_name
						if(name ~= nil)then
							material_translations[name] = ui_name
							for k, v in pairs(tags)do
								if(material_tags[name] ~= nil)then
									table.insert(material_tags[name], "[" .. k .. "]")
								else
									material_tags[name] = {}
									table.insert(material_tags[name], "[" .. k .. "]")
								end
							end
						end
					end
				end
			end
		end
	end
	parser:parse(ModTextFileGetContent("data/materials.xml"))
	for i, p in pairs(handler.root.Materials) do
		if i == "Reaction" then
			for i2, p2 in pairs(handler.root.Materials[i]) do
				if(p2._attr ~= nil)then
					local inputcount = 2
					new_reaction = {inputs={},outputs={}}
					if(p2._attr.input_cell1 ~= nil)then
						table.insert(new_reaction.inputs, p2._attr.input_cell1)
                    				end
                    				if(p2._attr.input_cell2 ~= nil)then
						table.insert(new_reaction.inputs, p2._attr.input_cell2)
					end
					if(p2._attr.input_cell3 ~= nil)then
						table.insert(new_reaction.inputs, p2._attr.input_cell3)
						inputcount = 3
					end
					if(p2._attr.output_cell1 ~= nil)then
						table.insert(new_reaction.outputs, p2._attr.output_cell1)
					end
					if(p2._attr.output_cell2 ~= nil)then
						table.insert(new_reaction.outputs, p2._attr.output_cell2)
					end
					if(p2._attr.output_cell3 ~= nil)then
						table.insert(new_reaction.outputs, p2._attr.output_cell3)
					end
					if(p2._attr.probability ~= nil)then
						new_reaction.probability = tonumber(p2._attr.probability)
					else
						new_reaction.probability = 100
					end

					if inputcount == 2 and p2._attr.input_cell1 ~= "air" and p2._attr.input_cell2 ~= "air" then
                    					existing_reactions[p2._attr.input_cell1 .. p2._attr.input_cell2] = new_reaction
					else
						--existing_reactions[p2._attr.input_cell1 .. p2._attr.input_cell2 .. p2._attr.input_cell3] = new_reaction --unused, couldn't figure out an efficient way of reacting more than 2 materials at a time
					end
				end
			end
		end
	end
	MY_EXTERN_GLOBAL.subtable = {existing_reactions, material_tags}
end
local reactions = MY_EXTERN_GLOBAL.subtable[1]
local material_tags = MY_EXTERN_GLOBAL.subtable[2]
iml = shuffle(IngestedMaterialsList())

local do_reaction = 0
for i=1,3 do
	if #iml < 2 then break end
	material_to_react = iml[1]
	other_material = iml[2]
	local reaction = nil

	local tag1 = "[]"
	local tag2 = "[]"
	if reactions[material_to_react .. other_material] ~= nil then					--Check if reaction exists for the material names
		reaction = reactions[material_to_react .. other_material]
	elseif reactions[other_material .. material_to_react] ~= nil then					--flip names try again
		reaction = reactions[other_material .. material_to_react]
	end
	if material_tags[other_material] ~= nil and reaction == nil then					--If input material 2 has tags, check if any of the tags react with input material 1
		for othertag_index=1,10 do
			local othertag = material_tags[other_material][othertag_index]
			if othertag == nil then break end
			if reactions[material_to_react .. othertag] ~= nil then
				reaction = reactions[material_to_react .. othertag]
				tag2 = othertag
			elseif reactions[othertag .. material_to_react] ~= nil then
				reaction = reactions[othertag .. material_to_react]
				tag2 = othertag
			end
		end
	end
	if material_tags[material_to_react] ~= nil and reaction == nil then				--If input material 1 has tags, check if any of the tags react with input material 2
		for tag_index=1,10 do
			local tag = material_tags[material_to_react][tag_index]
			if tag == nil then break end
			if reactions[tag .. other_material] ~= nil then
				reaction = reactions[tag .. other_material]
				tag1 = tag
			elseif reactions[other_material .. tag] ~= nil then
				reaction = reactions[tag .. material_to_react]
				tag1 = tag
			end
		end
	end
	if material_tags[material_to_react] ~= nil and material_tags[other_material] ~= nil and reaction == nil then	--if both input materials have tags, check if any of the tags react with each other
		for tag_index=1,10 do
			local tag = material_tags[material_to_react][tag_index]
			if tag == nil then break end
			for othertag_index=1,10 do
				local othertag = material_tags[other_material][othertag_index]
				if othertag == nil then break end
				if reactions[tag .. othertag] ~= nil then
					reaction = reactions[tag .. othertag]
					tag1 = tag
					tag2 = othertag
				elseif reactions[othertag .. tag] ~= nil then
					reaction = reactions[othertag .. tag]
					tag1 = tag
					tag2 = othertag
				end
			end
		end
	end
	if reaction ~= nil then
		local amount_to_react = math.random(math.min(getMaterialCount(material_to_react), getMaterialCount(other_material)))
		local string = ""
		if reaction["outputs"] ~= nil then
			for l=1,3 do
				local output = reaction["outputs"][l] 
				if output ~= nil then
					if output ~= "air" then
						if output == tag1 then output = material_to_react elseif output == tag2 then output = other_material end
						AddMaterialInventoryMaterial(entity, output, getMaterialCount(output) + amount_to_react)
					end
					string = string .. output .. " + "
				end
			end
		end
		--GamePrint(material_to_react ..  " + " .. other_material .. " --> " .. string:sub(1, -3))
		AddMaterialInventoryMaterial(entity, material_to_react, getMaterialCount(material_to_react) - amount_to_react)
		AddMaterialInventoryMaterial(entity, other_material, getMaterialCount(other_material) - amount_to_react)
	end
end

--DIGESTION 

for material_index, material in pairs(reactable) do
	local mat_count = getMaterialCount(material)
	alc_count = getMaterialCount("alcohol")
	if mat_count > 0 then 
		if (( get_index( CellFactory_GetAllLiquids(), material ) ~= nil or alc_count > 0 or get_index( CellFactory_GetAllFires(), material ) ~= nil or get_index( CellFactory_GetAllGases(), material ) ~= nil) or string.match( material, "marinated") or ( math.random(10) == 1 ) ) and (getMaterialCount("vomit") < 1 and ComponentGetValue2(expelletory_status, "value_int") ~= 5) then --If we aren't sick
			if alc_count > 0 then AddMaterialInventoryMaterial(entity, "alcohol", alc_count - 1 ) end

			if not ((EntityHasTag( entity, "iron_stomach" ) or getMaterialCount("magic_liquid_protection_all") > 0) and healthiness[material_index] <= 0) then
				if healthiness[material_index] < 0 then
					EntityInflictDamage( entity, healthiness[material_index]*-1, "DAMAGE_MATERIAL", "ingesting " .. material, "NONE", 0, 0)
					crouch = true
				else
					EntityInflictDamage( entity, healthiness[material_index]*-1, "DAMAGE_HEALING", "ingesting " .. material, "NONE", 0, 0)
				end
			end

			if digests_to[material_index] ~= material then
				AddMaterialInventoryMaterial(entity, digests_to[material_index], getMaterialCount(digests_to[material_index]) + 1 )
				AddMaterialInventoryMaterial(entity, material, getMaterialCount(material) - 1)
				if not string.match( material, "water") and not string.match( material, "fire") then
					AddMaterialInventoryMaterial(entity, "acid_gas", getMaterialCount("acid_gas") + 10)
					if material == "beans" then AddMaterialInventoryMaterial(entity, "acid_gas", getMaterialCount("acid_gas") + 20) end
				end
				break
			end
		elseif getMaterialCount("vomit") > 0 or ComponentGetValue2(expelletory_status, "value_int") == 5 then --If we are sick
			if digests_to[material_index] == "poo" or digests_to[material_index] == "diarrhea" or digests_to[material_index] == "urine" or digests_to[material_index] == "vomit" then --If material would digest to pee or poo, turn it into vomit instead
				AddMaterialInventoryMaterial(entity, "vomit", getMaterialCount("vomit") + 1 )
				AddMaterialInventoryMaterial(entity, material, getMaterialCount(material) - 1)
			end
		end
	end
end

-----------------------------------------------------------------------------------------
--DEFECATION
-----------------------------------------------------------------------------------------
if (getMaterialCount({"poo","diarrhea"})  > 100 and ComponentGetValue2( controls, "mButtonDownInteract")) or (ComponentGetValue2(expelletory_status, "value_int") == 2) or getMaterialCount("diarrhea") > 100 then

	--poo = EntityLoad("mods/ir/files/turd.xml", x, y) --A dynamic turd, couldn't get it to look right though

	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	if material_inventory == nil then return end
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")
	local empty = 1
	crouch = true

	for k,material_amount in pairs(count_per_material_type) do
		if material_amount ~= 0 then --material exists in the inventory
                   		material_name = CellFactory_GetName(k-1)
			if (get_index(CellFactory_GetAllLiquids( false, false ), material_name) == nil or material_name == "diarrhea") and material_name ~= nil then
				local count = math.random(0, material_amount)
				PhysicsApplyForce( entity, -count, count )
				ComponentSetValue2(poocomp, "create_real_particles", true)
				ComponentSetValue2(poocomp, "x_pos_offset_min", 0)
				ComponentSetValue2(poocomp, "x_pos_offset_max", 0)
				ComponentSetValue2(poocomp, "y_pos_offset_min", 0)
				ComponentSetValue2(poocomp, "y_pos_offset_max", 0)
				ComponentSetValue2(poocomp, "b2_force", count)
				ComponentSetValue2(poocomp, "x_vel_max", 0)
				ComponentSetValue2(poocomp, "x_vel_min", -20 - (count*0.1))
				ComponentSetValue2(poocomp, "y_vel_max", count*0.1)
				ComponentSetValue2(poocomp, "y_vel_min", 0)
				ComponentSetValue2(poocomp, "emitted_material_name", material_name)
				ComponentSetValue2(poocomp, "count_min", math.floor(count*0.5))
				ComponentSetValue2(poocomp, "count_max", math.floor(count*0.5))
				AddMaterialInventoryMaterial(entity, material_name, material_amount - count)
				if ComponentGetValue2(expelletory_status, "value_int") ~= 2 then
					ComponentSetValue2(expelletory_status, "value_int", 2)
					GamePlaySound( "data/audio/Desktop/projectiles.bank", "player_projectiles/bullet_slime/create", x, y )
				end
				empty = 0 
				break
			end
		end
	end
	if empty == 1 then
		ComponentSetValue2(expelletory_status, "value_int", 0)
	end
else
	ComponentSetValue2(poocomp, "count_min", 0)
	ComponentSetValue2(poocomp, "count_max", 0)
end
-----------------------------------------------------------------------------------------
--FARTS
-----------------------------------------------------------------------------------------
local fartable = getMaterialCount("gases")+getMaterialCount("fires")
if ((fartable > 1000 and math.random(1+math.floor(10000/fartable)) == 1) or (ComponentGetValue2(expelletory_status, "value_int") == 4)) and ComponentGetValue2(expelletory_status, "value_int") ~= 5 or (fartable > 0 and ComponentGetValue2( controls, "mButtonDownInteract"))then

	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	if material_inventory == nil then return end
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")
	x2,y2 = ComponentGetValue2( controls, "mAimingVector")
	local factor = 1
	if (x2 > 0 and entity_vel_x < 0) or (x2 < 0 and entity_vel_x > 0) then factor = -1 end
	local count = math.random(0, 100)

	for k,material_amount in pairs(count_per_material_type) do
		if material_amount ~= 0 then --material exists in the inventory
            material_name = CellFactory_GetName(k-1)
			if get_index(CellFactory_GetAllGases(true, false), material_name) ~= nil or get_index(CellFactory_GetAllFires(true, false), material_name) ~= nil then
				if material_name ~= "acid_gas" then
					ComponentSetValue2(fartcomp, "emitted_material_name", material_name)
					ComponentSetValue2(fartcomp, "x_vel_min", -40+(math.abs(entity_vel_x)*50*factor))
					ComponentSetValue2(fartcomp, "x_vel_max", -30+(math.abs(entity_vel_x)*50*factor))
					ComponentSetValue2(fartcomp, "y_vel_max", 10+(entity_vel_y*10))
					ComponentSetValue2(fartcomp, "y_vel_min", 0+(entity_vel_y*10))
					ComponentSetValue2(fartcomp, "count_min", count)
					ComponentSetValue2(fartcomp, "count_max", count)
				end
				ComponentSetValue2(poocomp, "emitted_material_name", material_name)
				ComponentSetValue2(poocomp, "create_real_particles", true)
				ComponentSetValue2(poocomp, "x_pos_offset_min", -1)
				ComponentSetValue2(poocomp, "x_pos_offset_max", 1)
				ComponentSetValue2(poocomp, "y_pos_offset_min", -1)
				ComponentSetValue2(poocomp, "y_pos_offset_max", 1)
				ComponentSetValue2(poocomp, "count_min", count)
				ComponentSetValue2(poocomp, "count_max", count)
				AddMaterialInventoryMaterial(entity, material_name, getMaterialCount(material_name) - count)
				if ComponentGetValue2(expelletory_status, "value_int") ~= 4 then
					ComponentSetValue2(expelletory_status, "value_int", 4)
					GamePlaySound( "data/audio/Desktop/projectiles.bank", "player_projectiles/bullet_burst_of_air/create", x, y )
				end
				if getMaterialCount(material_name) < 1 then
					AddMaterialInventoryMaterial(entity, material_name, 0)
				end
				break
			end
		end
	end
	if fartable < 1 or math.random(100) == 1 then ComponentSetValue2(expelletory_status, "value_int", 0) end
else
	ComponentSetValue2(fartcomp, "count_min", 0)
	ComponentSetValue2(fartcomp, "count_max", 0)
end

-----------------------------------------------------------------------------------------
--URINATION
-----------------------------------------------------------------------------------------
if getMaterialCount("urine") > 0 and ComponentGetValue2( controls, "mButtonDownInteract") then
	x2,y2 = ComponentGetValue2( controls, "mAimingVector")
	interp = getMaterialCount("urine")^2 * 0.01
	curve = ((getMaterialCount("urine")^2 * 0.1)+(getMaterialCount("urine")^0.5 * interp)) / interp + 1
	vel_x = (x2*(0.015*curve))+(math.random(-10,10)*0.15)
	vel_y = (y2*(0.015*curve))+(math.random(-10,10)*0.15)
	--shoot_projectile( entity, "mods/ir/files/material_urine.xml", x, y, vel_x, vel_y )

	local factor = 1
	if (x2 > 0 and entity_vel_x < 0) or (x2 < 0 and entity_vel_x > 0) then factor = -1 end

	ComponentSetValue2(peecomp, "x_vel_max", math.abs(vel_x)+(math.abs(entity_vel_x)*50*factor))
	ComponentSetValue2(peecomp, "x_vel_min", math.abs(vel_x)+(math.abs(entity_vel_x)*50*factor))
	ComponentSetValue2(peecomp, "y_vel_max", vel_y+entity_vel_y)
	ComponentSetValue2(peecomp, "y_vel_min", vel_y+entity_vel_y)
	ComponentSetValue2(peecomp, "count_min", 1)
	ComponentSetValue2(peecomp, "count_max", 1)
	AddMaterialInventoryMaterial(entity, "urine", getMaterialCount("urine") - 1)
	--GamePlaySound( "data/audio/Desktop/materials.bank", "materials/spray_potion", x, y )
else
	ComponentSetValue2(peecomp, "count_min", 0)
	ComponentSetValue2(peecomp, "count_max", 0)
end
-----------------------------------------------------------------------------------------
--UPHEAVAL
-----------------------------------------------------------------------------------------
--vomit_gui = vomit_gui or GuiCreate()
--GuiStartFrame(vomit_gui)
--local button = GuiImageButton( vomit_gui, 3, 597, 230, "", "mods/ir/files/feather_hand.png" ) 
--if button == true then
--	ComponentSetValue2(expelletory_status, "value_int", 5)
--	GamePlaySound( "data/audio/Desktop/misc.bank", "game_effect/food_poisoning/create", x, y )
--end
--GuiTooltip( vomit_gui, "Induce Vomiting", "" )

EntityRemoveIngestionStatusEffect( entity, "FOOD_POISONING" )
if getMaterialCount("vomit") > 0 then
	local effect_amount = getMaterialCount("vomit")+1
	EntityIngestMaterial( entity, CellFactory_GetType("vomit"), effect_amount ) 
	AddMaterialInventoryMaterial(entity, "vomit", getMaterialCount("vomit") - effect_amount)
end

local vomitable_count = getMaterialCount("all") - (getMaterialCount("poo")+getMaterialCount("urine")+getMaterialCount("diarrhea")+getMaterialCount("gases"))
if ((getMaterialCount("vomit") > 10 and math.random(1+math.floor(10000/getMaterialCount("vomit"))) == 1) or (ComponentGetValue2(expelletory_status, "value_int") == 3) or (ComponentGetValue2(expelletory_status, "value_int") == 5)) and not EntityHasTag( entity, "iron_stomach" ) then
	if ((getMaterialCount("vomit") < 1 or math.random(100) == 1) and ComponentGetValue2(expelletory_status, "value_int") ~= 5) or vomitable_count < 1 then 
		ComponentSetValue2(expelletory_status, "value_int", 0) 
	end
	crouch = true
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	if material_inventory == nil then return end
	local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")
	x2,y2 = ComponentGetValue2( controls, "mAimingVector")
	local offset = 3.3
	if x2 > 0 then offset = 2.3 end
	ComponentSetValue2(vomitcomp, "offset", offset,-1)
	local factor = 1
	if (x2 > 0 and entity_vel_x < 0) or (x2 < 0 and entity_vel_x > 0) then factor = -1 end

	material_name = IngestedMaterialsList()[math.random(#IngestedMaterialsList())]
	particle_material = material_name
	material_amount = getMaterialCount(material_name)
	if string.find(material_name, "meat") ~= nil and string.find(material_name, "sand") == nil then particle_material = "meat_slime_sand" end

	if (material_name ~= "poo" and material_name ~= "diarrhea" and material_name ~= "urine" and get_index(CellFactory_GetAllGases(true, false), material_name) == nil) then
		local min = {5,material_amount}
		table.sort(min)
		local count = math.random(0, min[1])
		if material_name == "acid_gas" then count = math.random(0, material_amount) end
		ComponentSetValue2(vomitcomp, "emitted_material_name", particle_material)
		ComponentSetValue2(vomitcomp, "count_min", count)
		ComponentSetValue2(vomitcomp, "count_max", count)
		ComponentSetValue2(vomitcomp, "x_vel_min", 15+(math.abs(entity_vel_x)*60*factor))
		ComponentSetValue2(vomitcomp, "x_vel_max", 25+(math.abs(entity_vel_x)*60*factor))
		AddMaterialInventoryMaterial(entity, material_name, material_amount - count)
		if ComponentGetValue2(expelletory_status, "value_int") ~= 3 and ComponentGetValue2(expelletory_status, "value_int") ~= 5 then
			GamePlaySound( "data/audio/Desktop/misc.bank", "game_effect/food_poisoning/create", x, y )
			ComponentSetValue2(expelletory_status, "value_int", 3)
		end
		if getMaterialCount(material_name) < 1 then
			AddMaterialInventoryMaterial(entity, material_name, 0)
		end
	end
else
	ComponentSetValue2(vomitcomp, "count_min", 0)
	ComponentSetValue2(vomitcomp, "count_max", 0)
end
-----------------------------------------------------------------------------------------
--GUI ICONS
-----------------------------------------------------------------------------------------

local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
if material_inventory == nil then return end
local count_per_material_type = ComponentGetValue2( material_inventory, "count_per_material_type")

local list = ""
for k,v in pairs(count_per_material_type) do
	if v ~= 0 then --material exists in the inventory
                   	material_name = GameTextGetTranslatedOrNot(CellFactory_GetUIName(k-1))
                    	material_amount = v

                    	if material_amount > 0 then
			list = list .. material_name .. "(" .. material_amount .. ") \n"
                    	end
	end
end   

local cell_count = getMaterialCount("all")
local solid_count = getMaterialCount("solids")
local liquid_count = getMaterialCount("liquids")
almost_explode = false
rand_x, rand_y = 0, 0
col_rand_x, col_rand_y = 0, 0
blad_rand_x, blad_rand_y = 0, 0

local pressure_tier = "satiation_00"

if EntityHasTag( entity, "iron_stomach" )  or (getMaterialCount("magic_liquid_protection_all") > 0 and getMaterialCount("vomit") < 1) then
	pressure_tier = "iron_stomach"
elseif effects[23] >= 0.15 then 
	--nothing
elseif cell_count > 3000 then 
	local material_inventory = EntityGetFirstComponentIncludingDisabled(entity, "MaterialInventoryComponent")
	ComponentSetValue2( material_inventory, "on_death_spill", true)
	if ComponentGetValue2( dmc, "hp") > 0.08 then ComponentSetValue2( dmc, "hp", 0.08) end
	EntityInflictDamage( entity, 1, "DAMAGE_OVEREATING", "overeating", "NORMAL", x, y)
	explosion = EntityLoad( "data/entities/misc/overeating_explosion.xml", x, y )
	EntityRemoveComponent( entity, EntityGetFirstComponentIncludingDisabled( entity, "MaterialInventoryComponent"))
	EntityAddComponent( entity, "MaterialInventoryComponent", {_tags="ingestion",})
elseif cell_count > 1750 then 
	almost_explode = true
	pressure_tier = "ingestion_damage"
	rand_x, rand_y = math.floor(math.random(0,cell_count-1750)/50), math.floor(math.random(0,cell_count-1750)/50)
elseif cell_count > 1500 then 
	pressure_tier = "satiation_06"
elseif cell_count > 1250 then 
	pressure_tier = "satiation_05"
elseif cell_count > 1000 then 
	pressure_tier = "satiation_04"
elseif cell_count > 750 then 
	pressure_tier = "satiation_03"
elseif cell_count > 500 then 
	pressure_tier = "satiation_02"
elseif cell_count > 250 then 
	pressure_tier = "satiation_01"
end
stomach_gui = stomach_gui or GuiCreate()
GuiStartFrame(stomach_gui)
GuiImage( stomach_gui, 2, 600+rand_x, 200+rand_y, "data/ui_gfx/status_indicators/" .. pressure_tier .. ".png", 1, 1, 1, 0, 0, "")
local stomach_text = (math.floor(cell_count/10) .. "%      ")
local stomach_offset = GuiGetTextDimensions( stomach_gui, stomach_text, 1, 2 )
local button = GuiButton( stomach_gui, 3, 620-stomach_offset, 200, stomach_text )
GuiTooltip( stomach_gui, "STOMACH", list )
if button == true then
	ComponentSetValue2(expelletory_status, "value_int", 5)
	GamePlaySound( "data/audio/Desktop/misc.bank", "game_effect/food_poisoning/create", x, y )
end

local charcomp = EntityGetFirstComponentIncludingDisabled( entity, "CharacterDataComponent" )
local vx, vy = ComponentGetValue2( charcomp, "mVelocity")
local max = {0.001,1+(cell_count*((cell_count*0.0002)*0.0003))}
table.sort(max)
local divisor = max[2]
if vy < 0 then --if moving up
	ComponentSetValue2( charcomp, "mVelocity", vx/divisor, vy/(divisor^3))
else
	ComponentSetValue2( charcomp, "mVelocity", vx/divisor, vy)
end

if almost_explode then 
	warning_gui = warning_gui or GuiCreate()
	GuiText( warning_gui, 270, 290, "Press [E] to expel waste" )
end
if crouch == true then
	local char = EntityGetFirstComponentIncludingDisabled( entity, "CharacterDataComponent")
	if ComponentGetValue2( char, "mFramesOnGround") > 3 then --is on ground?
		x2,y2 = ComponentGetValue2( controls, "mAimingVector")
		if math.abs(entity_vel_x) < 0.2 then
			if ComponentGetValue2(controls, "mButtonDownDown") then
				GamePlayAnimation( entity, "eat", 10)
			else 
				GamePlayAnimation( entity, "stand_crouched", 10) 
			end
		else
			if (x2/math.abs(x2)) == (entity_vel_x/math.abs(entity_vel_x)) then --if entity is facing the direction it's moving in
				GamePlayAnimation( entity, "walk_crouched", 10) 
			else
				GamePlayAnimation( entity, "walk_backwards_crouched", 10) 
			end
		end
	else
		GamePlayAnimation( entity, "walk_backwards_crouched", 10) 
	end
	ComponentSetValue2(crouching, "value_bool", true)
elseif ComponentGetValue2(crouching, "value_bool") == true then
	EntityRemoveComponent( entity, EntityGetFirstComponentIncludingDisabled( entity, "SpriteAnimatorComponent"))
	EntityAddComponent( entity, "SpriteAnimatorComponent", {})
	ComponentSetValue2(crouching, "value_bool", false)
end
